package gabriel.proyecto;

import gabriel.proyecto.imagenes.Punto;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xmlpull.v1.XmlSerializer;

import android.content.Context;
import android.util.Xml;

public class XmlProyecto {

	/**
	 * @uml.property name="nombre"
	 */
	private String nombre;
	/**
	 * @uml.property name="silueta"
	 */
	private String silueta;
	/**
	 * @uml.property name="fecha"
	 */
	private String fecha;
	

	/**
	 * @uml.property name="direccion"
	 */
	private String direccion;
	/**
	 * @uml.property name="serializer"
	 * @uml.associationEnd
	 */
	XmlSerializer serializer;
	/**
	 * @uml.property name="fileos"
	 */
	FileOutputStream fileos;
	/**
	 * @uml.property name="documento"
	 * @uml.associationEnd
	 */
	Document documento;

	/**
	 * @uml.property name="descripcion"
	 */
	private String descripcion;
	
	static private String[] Siluetas = { "Coche normal", "Van", "Camioneta",
			"Minibus", "Camion" };
	static private String[] Zonas = { "Piloto", "Copiloto",
			"Asiento izquierdo", "Asiento derecho", "Central" };

	public XmlProyecto(File archivo) throws Exception {


		this.direccion = archivo.getCanonicalPath();
		fileos = new FileOutputStream(archivo);

		// creamos un XMLSerializer para escribir el proyecto mas adelante
		serializer = Xml.newSerializer();

		this.silueta = null;
	}

	public XmlProyecto() {

	}

	/**
	 * @return
	 * @uml.property name="nombre"
	 */
	public String getNombre() {
		return this.nombre;
	}

	/**
	 * @return
	 * @uml.property name="fecha"
	 */
	public String getFecha() {
		return this.fecha;
	}

	/**
	 * @return
	 * @uml.property name="silueta"
	 */
	public String getSilueta() {
		return this.silueta;
	}
	

	//Carga los puntos de un proyecto existente, sino devuelve un array vacio
	@SuppressWarnings("rawtypes")
	public ArrayList cargarPuntos(Context context) {

		try {
			NodeList posicionesX = documento.getElementsByTagName("X");
			NodeList posicionesY = documento.getElementsByTagName("Y");
			NodeList Zonas = documento.getElementsByTagName("NumZona");
			NodeList Gravedades = documento.getElementsByTagName("Gravedad");
			NodeList Fotos = documento.getElementsByTagName("Foto");

			Punto p;
			ArrayList<Punto> puntos = new ArrayList<Punto>();
			float X;
			float Y;
			int zona;
			int gravedad;
			int i;
			String foto;

			for (i = 0; i < posicionesX.getLength(); i++) {
				X = Float.parseFloat(posicionesX.item(i).getTextContent());
				Y = Float.parseFloat(posicionesY.item(i).getTextContent());
				zona = Integer.parseInt(Zonas.item(i).getTextContent());
				gravedad = Integer
						.parseInt(Gravedades.item(i).getTextContent());
				foto = Fotos.item(i).getTextContent();

				boolean b;
				b = (!(foto.equals("")));
				p = new Punto(context, X, Y, gravedad, zona, b);
				if (b) {
					p.setDireccionFoto(foto);
				}

				puntos.add(p);
			}
			return puntos;
		} catch (Exception ex) {
			return (new ArrayList<Punto>());
		}
	}

	//abre un fichero Xml existente a partir de la direccion
	public boolean abrirXml(String direccion) {
		try {

			this.direccion = direccion;
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();

			DocumentBuilder builder = factory.newDocumentBuilder();
			FileInputStream filename = new FileInputStream(direccion);

			this.documento = builder.parse(filename);
			this.documento.normalize();
			NodeList nodos = documento.getElementsByTagName("Nombre");
			this.nombre = nodos.item(0).getTextContent();

			nodos = documento.getElementsByTagName("Fecha");
			this.fecha = nodos.item(0).getTextContent();

			nodos = documento.getElementsByTagName("nombreSilueta");

			this.silueta = nodos.item(0).getTextContent();

			
			try {
				this.descripcion = documento
						.getElementsByTagName("DescripcionProy").item(0)
						.getTextContent();
			} catch (Exception e) {
				this.descripcion = "";
			}
			return true;
		} catch (Exception ex) {

			return false;
		}

	}

	/**
	 * @return
	 * @uml.property name="descripcion"
	 */
	public String getDescripcion() {
		return descripcion;
	}

	//crea un nuevo proyecto
	public void escribirProyecto(String nombre, String silueta)
			throws Exception {

		serializer.setOutput(fileos, "UTF-8");
		serializer.startDocument(null, Boolean.valueOf(true));
		serializer.setFeature(
				"http://xmlpull.org/v1/doc/features.html#indent-output", true);
		serializer.startTag(null, "Proyecto");
		
		serializer.attribute(null, "Nombre", nombre);

		serializer.startTag(null, "Nombre");
		serializer.text(nombre);
		serializer.endTag(null, "Nombre");
		serializer.startTag(null, "Fecha");
		
		Date date = Calendar.getInstance().getTime();
		SimpleDateFormat df= new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss",Locale.ENGLISH);
	
		
		serializer.text(df.format(date));
		serializer.endTag(null, "Fecha");
		serializer.startTag(null, "Silueta");
		serializer.startTag(null, "nombreSilueta");
		serializer.text(silueta);
		serializer.endTag(null, "nombreSilueta");

		serializer.startTag(null, "descripcion");
		String numeroSt = silueta.replace("sample_", "");
		int aux = Integer.parseInt(numeroSt);
		serializer.text(Siluetas[aux - 1]);
		serializer.endTag(null, "descripcion");
		serializer.endTag(null, "Silueta");
		serializer.startTag(null, "Puntos");
		serializer.endTag(null, "Puntos");
		
		serializer.endTag(null, "Proyecto");

		serializer.endDocument();
		// escribe los datos en el FileOutputStream
		serializer.flush();

		fileos.close();

		this.nombre = nombre;
		this.silueta = silueta;
		this.fecha = date.toGMTString();
		this.descripcion = null;
	}

	//Aade un punto a un proyecto
	public boolean addPunto(Punto p) {

		try {

			NodeList nodos = documento.getElementsByTagName("Puntos");

			Node nodoPuntos = nodos.item(0);

			Element punto = documento.createElement("Punto");

			// Aado la posicion del punto
			Element posicion = documento.createElement("Posicion");
			// Posicion X
			Element X = documento.createElement("X");
			X.appendChild(documento.createTextNode(Float.toString(p.getX())));
			// Posicion Y
			Element Y = documento.createElement("Y");
			Y.appendChild(documento.createTextNode(Float.toString(p.getY())));

			posicion.appendChild(X);
			posicion.appendChild(Y);
			punto.appendChild(posicion);

			// Aado la zona
			Element Zona = documento.createElement("Zona");
			Zona.appendChild(documento.createTextNode(Integer.toString(p
					.getZona())));

			Element numZona = documento.createElement("NumZona");
			numZona.appendChild(documento.createTextNode(Integer.toString(p
					.getZona())));
			Element descripcion = documento.createElement("Descripcion");
			descripcion
					.appendChild(documento.createTextNode((Zonas[p.getZona()])));
			Zona.appendChild(numZona);
			Zona.appendChild(descripcion);
			punto.appendChild(Zona);

			// Aado la gravedad
			Element Gravedad = documento.createElement("Gravedad");
			Gravedad.appendChild(documento.createTextNode(Integer.toString(p
					.getGravedad())));
			punto.appendChild(Gravedad);

			Element Foto = documento.createElement("Foto");
			// Aado la foto si tiene
			if (p.tieneFoto()) {
				// f.getImageView().get
				Foto.appendChild(documento.createTextNode(p.getDireccionFoto()));

			}
			punto.appendChild(Foto);
			// Por ultimo aado el punto al nodo PUNTOS
			nodoPuntos.appendChild(punto);

			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer();

			transformer.setOutputProperty(OutputKeys.INDENT, "yes");

			StreamResult result = new StreamResult(new File(direccion));
			DOMSource dom = new DOMSource(documento);
			transformer.transform(dom, result);

			return true;
		} catch (Exception ex) {
			return false;
		}

	}
	//elimina un punto existente
	public boolean eliminarPunto(Punto p) {
		try {
			NodeList Puntos = documento.getElementsByTagName("Punto");
			NodeList posicionesX = documento.getElementsByTagName("X");
			NodeList posicionesY = documento.getElementsByTagName("Y");
			boolean borrado = false;
			float X, Y;
			for (int i = 0; i < Puntos.getLength(); i++) {
				X = Float.parseFloat(posicionesX.item(i).getTextContent());
				Y = Float.parseFloat(posicionesY.item(i).getTextContent());
				if ((p.getX() == X) && (p.getY() == Y)) {
					Puntos.item(i).getParentNode().removeChild(Puntos.item(i));
					// documento.removeChild(Puntos.item(i));

					Transformer transformer = TransformerFactory.newInstance()
							.newTransformer();

					transformer.setOutputProperty(OutputKeys.INDENT, "yes");
					StreamResult result = new StreamResult(new File(direccion));
					DOMSource dom = new DOMSource(documento);
					transformer.transform(dom, result);
					borrado = true;
				}
			}
			return borrado;
		} catch (Exception ex) {
			return false;
		}

	}

	//Devuelve el punto con coordenadas posX y posY
	public Punto getPunto(float posX, float posY) {
		NodeList Puntos = documento.getElementsByTagName("Punto");
		NodeList posicionesX = documento.getElementsByTagName("X");
		NodeList posicionesY = documento.getElementsByTagName("Y");
		NodeList Zonas = documento.getElementsByTagName("NumZona");
		NodeList Gravedades = documento.getElementsByTagName("Gravedad");
		NodeList Fotos = documento.getElementsByTagName("Foto");
		int zona, gravedad;
		float X, Y;
		String direccion;
		for (int i = 0; i < Puntos.getLength(); i++) {
			X = Float.parseFloat(posicionesX.item(i).getTextContent());
			Y = Float.parseFloat(posicionesY.item(i).getTextContent());
			if ((X == posX) && (Y == posY)) {

				zona = Integer.parseInt(Zonas.item(i).getTextContent());
				gravedad = Integer
						.parseInt(Gravedades.item(i).getTextContent());
				direccion = Fotos.item(i).getTextContent();

				boolean b = (!(direccion.equals("")));
				Punto p = new Punto(null, posX, posY, gravedad, zona, b);
				p.setDireccionFoto(direccion);
				return p;
			}
		}
		return null;
	}

	//Edita el punto guiandose por las coordenadas que no pueden cambiar
	public boolean editarPunto(Punto punto) {
		try {
			NodeList Puntos = documento.getElementsByTagName("Punto");
			NodeList posicionesX = documento.getElementsByTagName("X");
			NodeList posicionesY = documento.getElementsByTagName("Y");
			// NodeList Zonas = documento.getElementsByTagName("NumZona");
			NodeList Gravedades = documento.getElementsByTagName("Gravedad");
			NodeList Fotos = documento.getElementsByTagName("Foto");
			float X, Y;

			for (int i = 0; i < Puntos.getLength(); i++) {
				X = Float.parseFloat(posicionesX.item(i).getTextContent());
				Y = Float.parseFloat(posicionesY.item(i).getTextContent());
				if ((X == punto.getX()) && (Y == punto.getY())) {

					// zona = Integer.parseInt(Zonas.item(i).getTextContent());
					Gravedades.item(i).setTextContent(
							Integer.toString(punto.getGravedad()));
					Fotos.item(i).setTextContent(punto.getDireccionFoto());

					Transformer transformer = TransformerFactory.newInstance()
							.newTransformer();

					transformer.setOutputProperty(OutputKeys.INDENT, "yes");

					StreamResult result = new StreamResult(new File(direccion));
					DOMSource dom = new DOMSource(documento);
					transformer.transform(dom, result);
					return true;

				}

			}
			return false;

		} catch (Exception ex) {
			return false;
		}

	}
	//escribe la descripcion en el proyecto
	public boolean escribirDescripcion(String des) {
		try {
			Node nodoProyecto = documento.getElementsByTagName("Proyecto")
					.item(0);
			try {
				// Si existe la descripcion la cambio
				Node Descripcion = documento.getElementsByTagName(
						"DescripcionProy").item(0);
				Descripcion.setTextContent(des);
			} catch (Exception ex) {
				// Si no existe creo el nodo y lo aado
				Element desc = documento.createElement("DescripcionProy");
				desc.appendChild(documento.createTextNode(des));
				nodoProyecto.appendChild(desc);

			}
			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer();

			transformer.setOutputProperty(OutputKeys.INDENT, "yes");

			StreamResult result = new StreamResult(new File(direccion));
			DOMSource dom = new DOMSource(documento);
			transformer.transform(dom, result);
			this.descripcion = des;
			return true;

		} catch (Exception ex) {
			return false;
		}

	}
	//escribe la posicion GPS en el proyecto
	public boolean escribirPosicion(String posicion){
		try {
			Node nodoProyecto = documento.getElementsByTagName("Proyecto")
					.item(0);
			try {
				// Si existe la descripcion la cambio
				Node Posicion = documento.getElementsByTagName(
						"PosicionGPS").item(0);
				Posicion.setTextContent(posicion);
			} catch (Exception ex) {
				// Si no existe creo el nodo y lo aado
				Element pos = documento.createElement("PosicionGPS");
				pos.appendChild(documento.createTextNode(posicion));
				nodoProyecto.appendChild(pos);

			}
			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer();

			transformer.setOutputProperty(OutputKeys.INDENT, "yes");

			StreamResult result = new StreamResult(new File(direccion));
			DOMSource dom = new DOMSource(documento);
			transformer.transform(dom, result);
			return true;

		} catch (Exception ex) {
			return false;
		}
	}

	//Escribe la fecha en el proyeco
	public boolean setFecha(Date fecha){
		try {
		
	
			Node Fecha = documento.getElementsByTagName(
						"Fecha").item(0);
			Fecha.setTextContent(fecha.toGMTString());
			

			
			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer();

			transformer.setOutputProperty(OutputKeys.INDENT, "yes");

			StreamResult result = new StreamResult(new File(direccion));
			DOMSource dom = new DOMSource(documento);
			transformer.transform(dom, result);
			this.fecha=fecha.toGMTString();
			return true;

		} catch (Exception ex) {
			return false;
		}
	}
	
}
